@@ -92,6 +92,20 @@ class gr_block {
 			    gr_vector_const_void_star &input_items,
 			    gr_vector_void_star &output_items) = 0;
 
+  /*!
+   * \brief Confirm that ninputs and noutputs is an acceptable combination.
+   *
+   * \param ninputs	number of input streams connected
+   * \param noutputs	number of output streams connected
+   *
+   * \returns true if this is a valid configuration for this block.
+   *
+   * This function is called by the runtime system whenever the
+   * topology changes.  Most classes do not need to override this.
+   * This check is in addition to the constraints specified by the input
+   * and output gr_io_signatures.
+   */
+  virtual bool check_topology (int ninputs, int noutputs);
 
   // ----------------------------------------------------------------
 
