@@ -56,11 +56,11 @@ class buffer_sizes (object):
 class flow_graph (basic_flow_graph):
     """add physical connection info to simple_flow_graph
     """
-    __slots__ = ['sorted_blocks', 'scheduler']
+    __slots__ = ['blocks', 'scheduler']
     
     def __init__ (self):
         basic_flow_graph.__init__ (self);
-        self.sorted_blocks = None
+        self.blocks = None
         self.scheduler = None
 
     def start (self):
@@ -68,9 +68,10 @@ class flow_graph (basic_flow_graph):
         if self.scheduler:
             raise RuntimeError, "Scheduler already running"
         self._setup_connections ()
+
         # cast down to gr_module_sptr
-        t = [x.block () for x in self.sorted_blocks]
-        self.scheduler = scheduler (t)
+        # t = [x.block () for x in self.topological_sort (self.blocks)]
+        self.scheduler = scheduler (self)
         self.scheduler.start ()
 
     def stop (self):
@@ -96,13 +97,13 @@ class flow_graph (basic_flow_graph):
     def _setup_connections (self):
         """given the basic flow graph, setup all the physical connections"""
         self.validate ()
-        self.sorted_blocks = self.topological_sort ()
+        self.blocks = self.all_blocks ()
         self._assign_details ()
         self._assign_buffers ()
         self._connect_inputs ()
 
     def _assign_details (self):
-        for m in self.sorted_blocks:
+        for m in self.blocks:
             ninputs = len (self.in_edges (m))
             noutputs = len (self.out_edges (m))
             m.set_detail (block_detail (ninputs, noutputs))
@@ -110,14 +111,14 @@ class flow_graph (basic_flow_graph):
     def _assign_buffers (self):
         """determine the buffer sizes to use, allocate them and attach to detail"""
         sizes = buffer_sizes (self)
-        for m in self.sorted_blocks:
+        for m in self.blocks:
             d = m.detail ()
             for index in range (d.noutputs ()):
                 d.set_output (index, sizes.allocate (m, index))
 
     def _connect_inputs (self):
         """connect all block inputs to appropriate upstream buffers"""
-        for m in self.sorted_blocks:
+        for m in self.blocks:
             d = m.detail ()
             for e in self.in_edges (m):
                 # FYI, sources don't have any in_edges
@@ -128,11 +129,13 @@ class flow_graph (basic_flow_graph):
                 d.set_input (our_port, buffer_add_reader (upstream_buffer))
     
 
-    def topological_sort (self):
-        """Return a topologically sorted list of vertices"""
-        # this is basically a depth-first search with checks
-        # for back edges (the non-DAG condition)
-        all_v = self.all_blocks ()
+    def topological_sort (self, all_v):
+        '''
+        Return a topologically sorted list of vertices.
+        This is basically a depth-first search with checks
+        for back edges (the non-DAG condition)
+
+        '''
 
         # it's correct without this sort, but this
         # should give better ordering for cache utilization
@@ -150,7 +153,7 @@ class flow_graph (basic_flow_graph):
     def _dfs_visit (self, u, output):
         # print "dfs_visit (enter): ", u
         u.ts_color = _GRAY
-        for v in [e.dst.block for e in self.out_edges(u)]:
+        for v in self.downstream_verticies (u):
             if v.ts_color == _WHITE:    # (u, v) is a tree edge
                 self._dfs_visit (v, output)
             elif v.ts_color == _GRAY:   # (u, v) is a back edge
@@ -175,3 +178,30 @@ class flow_graph (basic_flow_graph):
         # print "sorted: ", x
         return x
         
+    def partition_graph (self, all_v):
+        '''Return a list of lists of nodes that are connected.
+        The result is a list of disjoint graphs.
+        The sublists are topologically sorted.
+        '''
+        result = []
+        working_v = all_v[:]    # make copy
+        while working_v:
+            rv = self._reachable_verticies (working_v[0], working_v)
+            result.append (self.topological_sort (rv))
+            for v in rv:
+                working_v.remove (v)
+        return result
+        
+    def _reachable_verticies (self, start, all_v):
+        for v in all_v:
+            v.ts_color = _WHITE
+
+        self._reachable_dfs_visit (start)
+        return [v for v in all_v if v.ts_color == _BLACK]
+
+    def _reachable_dfs_visit (self, u):
+        u.ts_color = _BLACK
+        for v in self.adjacent_verticies (u):
+            if v.ts_color == _WHITE:
+                self._reachable_dfs_visit (v)
+        return None
